/*
 * Copyright (c) 2002/2003, Simon Brown
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *   - Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *   - Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in
 *     the documentation and/or other materials provided with the
 *     distribution.
 *
 *   - Neither the name of TagUnit nor the names of its contributors may
 *     be used to endorse or promote products derived from this software
 *     without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package org.tagunit;

/**
 * The superclass for all test contexts.
 *
 * @author    Simon Brown
 */
public class DefaultTestContext implements TestContext, Comparable {

  /** the name of this context */
  private String name = "";

  /** the message (optional) associated with this context */
  private String message = "";

  private TestContextStatus status = TestContext.UNDEFINED;

  /** the number of milliseconds taken to run the tests within this context */
  private long time;

  /** a flag to indicate the type of the test context */
  private TestContextType type = TestContext.MANUAL_TEST;

  /** the parent of this context */
  private TestContextContainer parent;

  /**
   * Default, no args constructor.
   */
  public DefaultTestContext() {
  }

  /**
   * Creates a new test context with the specified name.
   *
   * @param name    the name of this context
   */
  public DefaultTestContext(String name) {
    this.name = name;
  }

  /**
   * Gets the name of this context.
   *
   * @return  the name as a String
   */
  public String getName() {
    return this.name;
  }

  /**
   * Sets the name of this context.
   *
   * @param name    the new name
   */
  public void setName(String name) {
    this.name = name;
  }

  /**
   * Gets the message associated with this context.
   *
   * @return  the message as a String
   */
  public String getMessage() {
    return this.message;
  }

  /**
   * Sets the message associated with this test context.
   *
   * @param s   the new message
   */
  public void setMessage(String s) {
    this.message = s;
  }

  /**
   * Gets the result of the tests that were run within this test context.
   *
   * @return  true if the tests were successful, false otherwise
   */
  public boolean getResult() {
    return status.getResult();
  }

  /**
   * Gets the number of milliseconds taken to run the tests
   * within this context.
   *
   * @return  the number of milliseconds as a long
   */
  public long getTime() {
    return time;
  }

  /**
   * Sets the number of milliseconds taken to run the tests
   * within this context.
   *
   * @param time      the number of milliseconds as a long
   */
  public void setTime(long time) {
    this.time = time;
  }

  /**
   * Gets the status of this test context (i.e. whether it passed,
   * failed, has warnings, etc).
   *
   * @return  a TestContextStatus instance representing the status of
   *          this test context
   */
  public TestContextStatus getStatus() {
    if (TagUnitProperties.getInstance().getIgnoreWarnings() && this.status.equals(TestContext.WARNING)) {
      return TestContext.PASS;
    } else {
      return this.status;
    }
  }

  /**
   * Sets the status of this test context (i.e. whether it passed,
   * failed, has warnings, etc)
   *
   * @param status    a TestContextStatus instance representing the status of
   *                  this test context
   */
  public void setStatus(TestContextStatus status) {
    this.status = status;
  }

  /**
   * Gets the type of this test context (i.e. manual, automatic, etc).
   *
   * @return  a constant representing the type of the context
   */
  public TestContextType getType() {
    return type;
  }

  /**
   * Sets the type of this test context (i.e. manual, automatic, etc).
   *
   * @param type    a constant representing the type of the context
   * @see   org.tagunit.TestContext
   */
  public void setType(TestContextType type) {
    this.type = type;
  }

  /**
   * Gets a reference to the TestContextContainer that this context belongs to.
   *
   * @return  a TestContextContainer instance, or null if this is the root
   */
  public TestContextContainer getParent() {
    return this.parent;
  }

  /**
   * Sets the parent for this test context.
   *
   * @param ctx   a reference to a TestContextContainer
   */
  public void setParent(TestContextContainer ctx) {
    this.parent = ctx;
  }

  /**
   * Compares this object with another (from the Comparable interface).
   *
   * @param o   the Object to compare against
   * @return  a negative integer, zero, or a positive integer if this is
   *          "less than", "the same" or "greater than" the specified Object
   */
  public int compareTo(Object o) {
    if (o instanceof TestContext) {
      return getName().compareTo(((TestContext)o).getName());
    } else {
      throw new ClassCastException(o.getClass().getName() + " cannot be compared to TestContext");
    }
  }

  /**
   * Returns a string representation of this class for debugging purposes.
   *
   * @return    a String
   */
  public final String toString() {
    return getClass().getName() + " [" + name + " | " + getResult() + "]";
  }

}