/*
 * Copyright (c) 2002/2003, Simon Brown
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *   - Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *   - Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in
 *     the documentation and/or other materials provided with the
 *     distribution.
 *
 *   - Neither the name of TagUnit nor the names of its contributors may
 *     be used to endorse or promote products derived from this software
 *     without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package org.tagunit;

import java.io.Serializable;


/**
 * The interface for test contexts.
 *
 * @author    Simon Brown
 */
public interface TestContext extends Serializable {

  /** typesafe enum representing an automatically (framework) generated test */
  public static final TestContextType AUTOMATIC_TEST = new TestContextType();

  /** typesafe enum representing a manual (user) test */
  public static final TestContextType MANUAL_TEST = new TestContextType();

  /** typesafe enum representing the "passed" status */
  public static final TestContextStatus PASS = new PositiveTestContextStatus("Pass");

  /** typesafe enum representing the "passed with warnings" status */
  public static final TestContextStatus WARNING = new PositiveTestContextStatus("Warning");

  /** typesafe enum representing the "failed" status */
  public static final TestContextStatus FAIL = new NegativeTestContextStatus("Fail");

  /** typesafe enum representing the "error" status */
  public static final TestContextStatus ERROR = new NegativeTestContextStatus("Error");

  /** typesafe enum representing the "undefined" status */
  public static final TestContextStatus UNDEFINED = new NegativeTestContextStatus("Undefined");

  /**
   * Nested class representing the type of a test context.
   */
  public static class TestContextType implements Serializable {

    /**
     * Creates a new TestContextType - private so no other instances
     * can be created.
     */
    public TestContextType() {
    }

  }

  /**
   * Gets the name of this context.
   *
   * @return  the name as a String
   */
  public String getName();

  /**
   * Sets the name of this context.
   *
   * @param name    the new name
   */
  public void setName(String name);

  /**
   * Gets the message associated with this context.
   *
   * @return  the message as a String
   */
  public String getMessage();

  /**
   * Sets the message associated with this context.
   *
   * @param s   the new message
   */
  public void setMessage(String s);

  /**
   * Gets the result of the tests that were run within this test context.
   *
   * @return  true if the tests were successful, false otherwise
   */
  public boolean getResult();

  /**
   * Gets the number of milliseconds taken to run the tests
   * within this context.
   *
   * @return  the number of milliseconds as a long
   */
  public long getTime();

  /**
   * Sets the number of milliseconds taken to run the tests
   * within this context.
   *
   * @param time      the number of milliseconds as a long
   */
  public void setTime(long time);

  /**
   * Gets the status of this test context (i.e. whether it passed,
   * failed, has warnings, etc)
   *
   * @return  a TestContextStatus instance representing the status of
   *          this test context
   */
  public TestContextStatus getStatus();

  /**
   * Sets the status of this test context (i.e. whether it passed,
   * failed, has warnings, etc)
   *
   * @param status    a TestContextStatus instance representing the status of
   *                  this test context
   */
  public void setStatus(TestContextStatus status);

  /**
   * Gets the type of this test context (i.e. manual, automatic, etc).
   *
   * @return  a constant representing the type of the context
   */
  public TestContextType getType();

  /**
   * Sets the type of this test context (i.e. manual, automatic, etc).
   *
   * @param type    a constant representing the type of the context
   * @see   org.tagunit.TestContext
   */
  public void setType(TestContextType type);

  /**
   * Gets a reference to the TestContextContainer that this context belongs to.
   *
   * @return  a TestContextContainer instance, or null if this is the root
   */
  public TestContextContainer getParent();

  /**
   * Sets the parent for this test context.
   *
   * @param ctx   a reference to a TestContextContainer
   */
  public void setParent(TestContextContainer ctx);

}