/*
 * Copyright (c) 2002/2003, Simon Brown
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *   - Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *   - Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in
 *     the documentation and/or other materials provided with the
 *     distribution.
 *
 *   - Neither the name of TagUnit nor the names of its contributors may
 *     be used to endorse or promote products derived from this software
 *     without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package org.tagunit;

import java.util.Stack;

/**
 * A helper class for managing the stack of test contexts that are being used.
 *
 * @author    Simon Brown
 */
public class TestContextManager {

  /** the root test context in use */
  private TestContextContainer rootTestContext;

  /** a stack used to maintain the current test context hierarchy */
  private Stack testContexts = new Stack();

  /**
   * Creates a new manager for the specified tag library.
   *
   * @param ctx   the test context for the tag library that this class
   *              is to manage
   */
  public TestContextManager(TestContextContainer ctx) {
    setRootTestContext(ctx);
    ctx.setManager(this);
  }

  /**
   * Gets a reference to the test context for the tag library being tested.
   *
   * @return  a TagLibraryTestContext instance
   */
  TestContextContainer getRootTestContext() {
    return this.rootTestContext;
  }

  void setRootTestContext(TestContextContainer ctx) {
    this.rootTestContext = ctx;

    // and push the top level test context onto the stack
    testContexts.clear();
    testContexts.push(ctx);
  }

  /**
   * Gets a reference to the test context for the tag library being tested.
   *
   * @return  a TagLibraryTestContext instance
   */
  public TagLibraryTestContext getTagLibraryTestContext() {
    TestContext ctx = getCurrentTestContext();

    while (!(ctx instanceof TagLibraryTestContext) && ctx != null) {
      ctx = ctx.getParent();
    }

    return (TagLibraryTestContext)ctx;
  }

  /**
   * Gets the tag test context that is currently being used.
   *
   * @return  a TagTestContext instance
   */
  public TagTestContext getTagTestContext() {
    TestContext ctx = getCurrentTestContext();

    while (!(ctx instanceof TagTestContext) && ctx != null) {
      ctx = ctx.getParent();
    }

    return (TagTestContext)ctx;
  }

  /**
   * Gets the test context container that is currently being used.
   *
   * @return  a TestContextContainer instance
   */
  public TestContextContainer getCurrentTestContextContainer() {
    TestContext ctx = getCurrentTestContext();

    if (ctx instanceof TestContextContainer) {
      return (TestContextContainer)ctx;
    } else {
      return ctx.getParent();
    }
  }

  /**
   * Gets the test context that is currently being used.
   *
   * @return  a TestContext instance
   */
  TestContext getCurrentTestContext() {
    return (TestContext)testContexts.peek();
  }

  /**
   * Signals that a text context is now being used.
   *
   * @param ctx   the TestContext
   */
  public void startTestContext(TestContext ctx) {
    getCurrentTestContextContainer().add(ctx);
    testContexts.push(ctx);
  }

  /**
   * Signals that a text context is no longer being used.
   *
   * @param ctx   the TestContext
   */
  public void endTestContext(TestContext ctx) {
    TestContext testContext = (TestContext)testContexts.peek();

    if (testContext == ctx) {
      testContexts.pop();
    } else {
      System.out.println("Expected to end " + ctx.getName() + " but found " + testContext.getName());
      throw new RuntimeException("Test contexts are not properly stacked");
    }
  }

}