/*
 * Copyright (c) 2002/2003, Simon Brown
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *   - Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *   - Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in
 *     the documentation and/or other materials provided with the
 *     distribution.
 *
 *   - Neither the name of TagUnit nor the names of its contributors may
 *     be used to endorse or promote products derived from this software
 *     without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package org.tagunit;

import java.util.*;

import javax.servlet.jsp.PageContext;

/**
 * A wrapper in which to store a list of all the test JSPs that are to be
 * executed for a tag. Also, this wrapper class stores the URIs to the setup
 * and teardown JSPs, if present.
 */
public class TestResources {

  /** the name of the setup JSP */
  private static final String SETUP_RESOURCE = "setUp.jsp";

  /** the name of the teardown JSP */
  private static final String TEARDOWN_RESOURCE = "tearDown.jsp";

  /** the prefix used to identify test JSPs */
  private static final String TEST_PREFIX = "test";

  /** the JSP suffix */
  private static final String JSP_EXTENSION = ".jsp";

  /** the base URI from which the resources are located */
  private String baseUri;

  /** the URI of the setup JSP page */
  private String setUpUri;

  /** the URI of the teardown JSP page */
  private String tearDownUri;

  /** the collection of URIs representing the tests to be performed */
  private Collection resources;

  /**
   * Default, no args constructor.
   */
  public TestResources(String baseUri, PageContext pageContext) {
    this.baseUri = baseUri;
    resources = new ArrayList();
    init(pageContext);
  }

  /**
   * Helper method to find files matching the filename "test*.jsp", starting
   * from the specified URI.
   *
   * @param pageContext   a reference to the PageContext representing the
   *                      environment in which to find files
   */
  private void init(PageContext pageContext) {
    Collection resources = pageContext.getServletContext().getResourcePaths(baseUri);

    if (resources == null || resources.isEmpty()) {
      return;
    }

    Iterator it = resources.iterator();
    String uri;

    // for each file in the directory, check that it matches the
    // filename filter
    while (it.hasNext()) {
      uri = (String)it.next();

      // todo   remove this when OC4J 9.0.3.0.0 ServletContext.getResourcePaths() bug has been fixed
      //System.out.println(uri);

      // find the last "/" character
      int index = uri.lastIndexOf('/');
      String s = uri.substring(index + 1, uri.length());
      if (s.startsWith(TEST_PREFIX) && s.endsWith(JSP_EXTENSION)) {
        this.add(uri);
      } else if (s.equalsIgnoreCase(SETUP_RESOURCE)) {
        this.setSetUpUri(uri);
      } else if (s.equalsIgnoreCase(TEARDOWN_RESOURCE)) {
        this.setTearDownUri(uri);
      }
    }
  }

  /**
   * Gets the URI for the setup JSP.
   *
   * @return  a URI as a String
   */
  public String getSetUpUri() {
    return setUpUri;
  }

  /**
   * Sets the URI for the setup JSP.
   *
   * @param setUpUri    a URI as a String
   */
  public void setSetUpUri(String setUpUri) {
    this.setUpUri = setUpUri;
  }

  /**
   * Determines whether there is a setup JSP to run.
   *
   * @return  true if the setup URI has been set, false otherwise
   */
  public boolean hasSetUp() {
    return setUpUri != null;
  }

  /**
   * Gets the URI for the teardown JSP.
   *
   * @return  a URI as a String
   */
  public String getTearDownUri() {
    return tearDownUri;
  }

  /**
   * Sets the URI for the teardown JSP.
   *
   * @param tearDownUri   a URI as a String
   */
  public void setTearDownUri(String tearDownUri) {
    this.tearDownUri = tearDownUri;
  }

  /**
   * Determines whether there is a teardown JSP to run.
   *
   * @return  true if the teardown URI has been set, false otherwise
   */
  public boolean hasTearDown() {
    return tearDownUri != null;
  }

  /**
   * Adds a test URI to this set of test resources.
   *
   * @param uri   a URI as a String
   */
  private void add(String uri) {
    resources.add(uri);
  }

  /**
   * Gets a collection of all test resources.
   *
   * @return  a Collection instance containing Strings representing URIs
   */
  public Collection getResources() {
    return Collections.unmodifiableCollection(resources);
  }

}