/*
 * Copyright (c) 2002/2003, Simon Brown
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *   - Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *   - Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in
 *     the documentation and/or other materials provided with the
 *     distribution.
 *
 *   - Neither the name of TagUnit nor the names of its contributors may
 *     be used to endorse or promote products derived from this software
 *     without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package org.tagunit.tagext.test;

import java.io.File;
import java.util.*;

import javax.servlet.ServletContext;

import org.tagunit.*;

/**
 * Allows test*.jsp files (within the specified directory) containing tests
 * to be included and executed.
 *
 * @author    Simon Brown
 */
public class IncludeTestContextsTag extends TestContextTag {

  private static final String TEST_PREFIX = "test";
  private static final String JSP_EXTENSION = ".jsp";

  /** the URI of the test(s) to be included */
  private String uri;

  /**
   * Template method called during doStartTag(). Tags can override this to
   * perform any custom processing.
   *
   * @param testContext   the current test context container
   */
  protected void doStartTag(TestContextContainer testContext) {
    // get the real path corresponding to the specified path
    ServletContext ctx = pageContext.getServletContext();
    String realPath = ctx.getRealPath(uri);

    // setup the iterator to be used by searching the path for those
    // files matching the filename filter
    List fileEntries = new ArrayList();
    findFiles(uri, realPath, fileEntries);
    String testUri;

    DefaultTestContext includedTestContext;
    Iterator it = fileEntries.iterator();
    while (it.hasNext()) {
      testUri = (String)it.next();
      includedTestContext = new DefaultTestContext(
          "Inclusion of tests from " + testUri);
      getTestContextManager().startTestContext(includedTestContext);

      try {
        pageContext.include(testUri);
      } catch (Exception e) {
        includedTestContext.setStatus(TestContext.ERROR);
        includedTestContext.setMessage(e.getMessage());
      } finally {
        getTestContextManager().endTestContext(includedTestContext);
      }
    }
  }

  /**
   * Sets the URI to the location of the test(s) to be included.
   *
   * @param s   the URI as a String
   */
  public void setUri(String s) {
    this.uri = s;
  }

  /**
   * Helper (recursive) method to find files matching the specified
   * filename filter.
   */
  private void findFiles(String path, String realPath, List fileEntries) {

    // remove the trailing slash as this is not necessary
    if (path.endsWith("/")) {
      path = path.substring(0, path.length() - 1);
    }

    // create a reference to the specified real path, and then
    // get a list of all the files in the directory
    File file = new File(realPath);
    File files[] = file.listFiles();

    String uri;

    // for each file in the directory, check that it matches the
    // filename filter
    for (int i = 0; i < files.length; i++) {
      file = files[i];

      // check that the filename matches the filter and should be included
      if (file.getName().startsWith(TEST_PREFIX) && file.getName().endsWith(JSP_EXTENSION)) {
        // create a FileEntry object representing it
        uri = path + "/" + file.getName();

        fileEntries.add(uri);
      }
    }
  }

}