/*
 * Copyright (c) 2002/2003, Simon Brown
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *   - Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *   - Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in
 *     the documentation and/or other materials provided with the
 *     distribution.
 *
 *   - Neither the name of TagUnit nor the names of its contributors may
 *     be used to endorse or promote products derived from this software
 *     without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package org.tagunit.taginfo.jsp11;

import org.tagunit.taginfo.*;
import org.w3c.dom.*;

/**
 * A parser that turns an XML document into an in-memory representation of
 * the tag library descriptor.
 *
 * @author    Simon Brown
 */
public class Jsp11TagLibraryDescriptorParser extends TagLibraryDescriptorParser {

  /**
   * Default, no args constructor.
   */
  public Jsp11TagLibraryDescriptorParser() {
  }

  /**
   * Parses the supplied Document representing a TLD file.
   *
   * @param doc   an XML Document object
   * @return  a TagLibraryInfo object representing the TLD
   * @throws org.tagunit.taginfo.TagLibraryInfoParseException  if the Document can't be parsed
   */
  public TagLibraryInfo parse(Document doc) throws TagLibraryInfoParseException {
    TagLibraryInfo tagLibraryInfo = new TagLibraryInfo();

    parseTagLibraryInfo(doc, tagLibraryInfo);
    parseTags(doc, tagLibraryInfo);

    return tagLibraryInfo;
  }

  private void parseTagLibraryInfo(Document doc, TagLibraryInfo tagLibraryInfo) {
    Node node = doc.getDocumentElement();
    NodeList children = node.getChildNodes();
    Node child;
    for (int i = 0; i < children.getLength(); i++) {
      child = children.item(i);

      String nodeName = child.getNodeName();

      if (nodeName.equalsIgnoreCase("tlibversion")) {
        tagLibraryInfo.setVersion(getNodeValue(child));
      } else if (nodeName.equalsIgnoreCase("jspversion")) {
        tagLibraryInfo.setJspVersion(getNodeValue(child));
      } else if (nodeName.equalsIgnoreCase("shortname")) {
        tagLibraryInfo.setShortName(getNodeValue(child));
      } else if (nodeName.equalsIgnoreCase("uri")) {
        tagLibraryInfo.setUri(getNodeValue(child));
      } else if (nodeName.equalsIgnoreCase("info")) {
        tagLibraryInfo.setDescription(getNodeValue(child));
      }
    }
  }

  private void parseTags(Document doc, TagLibraryInfo tagLibraryInfo) throws TagLibraryInfoParseException {
    TagInfo tag;
    NodeList tagNodes = doc.getElementsByTagName("tag");
    for (int i = 0; i < tagNodes.getLength(); i++) {
      tag = createTag(tagNodes.item(i));
      try {
        tagLibraryInfo.add(tag);
      } catch (DuplicateTagException e) {
        // a duplicate tag was found in the tag library
        throw new TagLibraryInfoParseException("Duplicate tag called " + tag.getName() + " was found in the tag library");
      }
    }
  }

  private TagInfo createTag(Node node) {
    TagInfo tag = new TagInfo();

    NodeList children = node.getChildNodes();
    Node child;
    for (int i = 0; i < children.getLength(); i++) {
      child = children.item(i);

      String nodeName = child.getNodeName();

      if (nodeName.equalsIgnoreCase("name")) {
        tag.setName(getNodeValue(child));
      } else if (nodeName.equalsIgnoreCase("tagclass")) {
        tag.setTagHandlerClass(getNodeValue(child));
      } else if (nodeName.equalsIgnoreCase("teiclass")) {
        tag.setTagExtraInfoClass(getNodeValue(child));
      } else if (nodeName.equalsIgnoreCase("bodycontent")) {
        tag.setBodyContent(getNodeValue(child));
      } else if (nodeName.equalsIgnoreCase("info")) {
        tag.setDescription(getNodeValue(child));
      } else if (nodeName.equalsIgnoreCase("attribute")) {
        TagAttributeInfo attr = new TagAttributeInfo(child);
        tag.add(attr);
      }
    }

    return tag;
  }

}
