/*
 * Copyright (c) 2002/2003, Simon Brown
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *   - Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *   - Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in
 *     the documentation and/or other materials provided with the
 *     distribution.
 *
 *   - Neither the name of TagUnit nor the names of its contributors may
 *     be used to endorse or promote products derived from this software
 *     without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package org.tagunit.test;

import javax.servlet.http.Cookie;
import javax.servlet.http.HttpServletRequest;

import org.tagunit.TestContextContainer;
import org.tagunit.TestContext;

/**
 * Asserts that a named cookie exists, and optionally also asserts its value.
 *
 * @author    Simon Brown
 */
public class CookieTestPackage extends AbstractTestPackage {

  private String name;
  private String value;

  /**
   * Creates a new test package for the specified tag.
   */
  public CookieTestPackage(String name, String value) {
    this.name = name;
    this.value = value;
  }

  /**
   * Runs the tests that are part of this test package.
   *
   * @param testContext   the TestContext to which these tests are to be run
   */
  public void executeTest(TestContextContainer testContext) {
    CookieHelper helper = new CookieHelper();
    HttpServletRequest request = (HttpServletRequest)getPageContext().getRequest();
    Cookie cookie = helper.getCookie(request.getCookies(), name);

    if (cookie != null) {
      // we've found the cookie, so next check the value
      if (value != null) {
        if (!value.equals(cookie.getValue())) {
          testContext.setMessage("Value of cookie named " + name + " (" + cookie.getValue() + ") does equal " + value + ".");
          testContext.setStatus(TestContext.FAIL);
        }
      }
    } else {
      testContext.setMessage("Cookie named " + name + " does not exist.");
      testContext.setStatus(TestContext.FAIL);
    }
  }

  /**
   * Gets the name of this test package.
   *
   * @return  the name of this test package as a String
   */
  public String getName() {
    if (value == null) {
      return "Existence of cookie named " + name;
    } else {
      return "Existence of cookie named " + name + " (value=" + value + ")";
    }
  }

}

class CookieHelper {

  /**
   * Default, no arguments constructor.
   */
  public CookieHelper() {
  }

  /**
   * Gets a reference to a named cookie from an array of cookies.
   *
   * @param cookies   an array of cookies
   * @param name      the name of the cookie to find
   * @return    a reference to a Cookie, or null if the cookie couldn't be found
   */
  public Cookie getCookie(Cookie cookies[], String name) {

    for (int i = 0; i < cookies.length; i++) {
      if (cookies[i].getName().equals(name)) {
        return cookies[i];
      }
    }

    // we've got this far so the specified cookie wasn't found
    return null;
  }

}