/*
 * Copyright (c) 2002/2003, Simon Brown
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *   - Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *   - Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in
 *     the documentation and/or other materials provided with the
 *     distribution.
 *
 *   - Neither the name of TagUnit nor the names of its contributors may
 *     be used to endorse or promote products derived from this software
 *     without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package org.tagunit.test;

import org.tagunit.TestContextContainer;
import org.tagunit.TestContext;

/**
 * Wraps up the automatic tests for a tag handler.
 *
 * @author    Simon Brown
 */
public class ExceptionTestPackage extends AbstractTestPackage {

  /** the fully qualified name of the exception type */
  private String exception;

  /** the message */
  private String message;

  /**
   * Creates a new test package to check an exception.
   */
  public ExceptionTestPackage() {
  }

  /**
   * Creates a new test package to check an exception.
   *
   * @param exception   the fully qualified name of the exception class
   */
  public ExceptionTestPackage(String exception) {
    this.exception = exception;
  }

  /**
   * Creates a new test package to check an exception.
   *
   * @param exception   the fully qualified name of the exception class
   * @param message     the message
   */
  public ExceptionTestPackage(String exception, String message) {
    this.exception = exception;
    this.message = message;
  }

  /**
   * Runs the tests that are part of this test package.
   *
   * @param testContext   the TestContext to which these tests are to be run
   */
  public void executeTest(TestContextContainer testContext) {
    if (getThrowable() == null) {
      testContext.setStatus(TestContext.FAIL);
      testContext.setMessage("No exception was thrown");
      return;
    }

    // an exception was thrown and we're not interested in what it was
    if (exception == null) {
      testContext.setStatus(TestContext.PASS);
      testContext.setMessage(null);
      return;
    }

    // the exception type was specified
    try {
      Class c = Class.forName(exception);
      if (c.isAssignableFrom(getThrowable().getClass())) {
        testContext.setStatus(TestContext.PASS);
        testContext.setMessage(null);
      } else {
        testContext.setStatus(TestContext.FAIL);
        testContext.setMessage("Exception thrown was not of type " + exception);
        return;
      }
    } catch (ClassNotFoundException e) {
      testContext.setStatus(TestContext.FAIL);
      testContext.setMessage("Class for exception " + exception + " could not be found");
      return;
    }

    if (message != null) {
      if (!message.equals(getThrowable().getMessage())) {
        testContext.setStatus(TestContext.FAIL);
        testContext.setMessage("Expected message of " + message + ", but found " + getThrowable().getMessage());
        return;
      } else {
        testContext.setStatus(TestContext.PASS);
      }
    }
  }

}