/*
 * Copyright (c) 2002/2003, Simon Brown
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *   - Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *   - Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in
 *     the documentation and/or other materials provided with the
 *     distribution.
 *
 *   - Neither the name of TagUnit nor the names of its contributors may
 *     be used to endorse or promote products derived from this software
 *     without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package org.tagunit.test;

import org.tagunit.TestContextContainer;
import org.tagunit.TestContext;

/**
 * Base class for tests that compare two strings.
 *
 * @author    Simon Brown
 */
public abstract class StringComparisonTestPackage extends AbstractTestPackage {

  /** the actual generated content */
  protected String actualContent;

  /** the expected content */
  protected String expectedContent;

  /** a flag to indicate whether whitespace is being ignored in the comparison */
  protected boolean ignoreWhitespace;

  /**
   * Creates a new test package for the specified tag.
   *
   * @param actualContent   the content that was generated
   * @param expectedContent   the content that was expected
   * @param ignoreWhitespace    a flag indicating whether whitespace is being ignored
   */
  public StringComparisonTestPackage(String actualContent, String expectedContent,
                                     boolean ignoreWhitespace) {
    this.actualContent = actualContent;
    this.expectedContent = expectedContent;
    this.ignoreWhitespace = ignoreWhitespace;
  }

  /**
   * Runs the tests that are part of this test package.
   *
   * @param testContext   the TestContext to which these tests are to be run
   */
  public void executeTest(TestContextContainer testContext) {
    // are we ignoring whitespace?
    if (ignoreWhitespace) {
      actualContent = trim(actualContent);
      expectedContent = trim(expectedContent);
    }

    boolean result = compare(testContext);
    if (result) {
      testContext.setStatus(TestContext.PASS);
    } else {
      testContext.setStatus(TestContext.FAIL);
    }
  }

  /**
   * Performs the comparison between the two pieces of text.
   *
   * @param testContext   the TestContext to which the comparison is to be run
   * @return  true if the actual content is comparable with the expected content,
   *          false otherwise
   */
  protected abstract boolean compare(TestContextContainer testContext);

  /**
   * Helper method to trim a String of it's whitespace (including spaces, newlines, etc).
   *
   * @param content   the String to be trimmed of whitespace
   * @return  the trimmed String
   */
  protected String trim(String content) {

    if (content == null) {
      return null;
    }

    StringBuffer filteredContent = new StringBuffer("");

    for (int i = 0; i < content.length(); i++) {
      char c = content.charAt(i);

      if (!Character.isWhitespace(c)) {
        filteredContent.append(c);
      }
    }

    return filteredContent.toString();
  }

}