/*
 * Copyright (c) 2002/2003, Simon Brown
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *   - Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *   - Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in
 *     the documentation and/or other materials provided with the
 *     distribution.
 *
 *   - Neither the name of TagUnit nor the names of its contributors may
 *     be used to endorse or promote products derived from this software
 *     without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package org.tagunit.test;

import java.util.Collection;
import java.util.Iterator;

import javax.servlet.jsp.PageContext;

import org.tagunit.*;
import org.tagunit.taginfo.TagInfo;
import org.tagunit.taginfo.TagLibraryInfo;

/**
 * Wraps up the automatic tests for a complete tag library.
 *
 * @author    Simon Brown
 */
public class TagLibraryTestPackage extends AbstractTestPackage {

  /** information about the tag being tested */
  private TagLibraryInfo tagLibraryInfo;

  /** the base URI from which user defined tests can be found */
  private String baseUri;

  /** the name of the tag to be tested, if applicable */
  private String tag;

  /**
   * Creates a new test package for the specified tag library.
   *
   * @param tagLibraryInfo    a TagLibraryInfo instance
   */
  public TagLibraryTestPackage(TagLibraryInfo tagLibraryInfo) {
    this.tagLibraryInfo = tagLibraryInfo;
  }

  /**
   * Creates a new test package for the specified tag library.
   *
   * @param tagLibraryInfo    a TagLibraryInfo instance
   */
  public TagLibraryTestPackage(TagLibraryInfo tagLibraryInfo, String baseUri, String tag, PageContext pageContext) {
    super(pageContext);
    this.tagLibraryInfo = tagLibraryInfo;
    this.baseUri = baseUri;
    this.tag = tag;
  }

  /**
   * Runs the tests that are part of this test package.
   *
   * @param testContext   the TestContext to which these tests are to be run
   */
  public void executeTest(TestContextContainer testContext) {
    System.out.println("Running tests for " + testContext.getName());
    TagLibraryTestContext tagLibraryTestContext = (TagLibraryTestContext)testContext;
    Collection tags = tagLibraryInfo.getTags();

    Iterator it = tags.iterator();
    TagInfo tagInfo;
    while (it.hasNext()) {
      tagInfo = (TagInfo)it.next();

      if (tag != null && !tag.equals(tagInfo.getName())) {
        continue;
      }

      // create a top level context for this tag
      TagTestContext tagTestContext = tagLibraryTestContext.getTagTestContext(tagInfo);
      tagTestContext.setTagInfo(tagInfo);
      testContext.getManager().startTestContext(tagTestContext);

      TestPackage automaticTestPackage = new AutomaticTagHandlerTestPackage(tagInfo);
      automaticTestPackage.executeTest(tagTestContext);

      if (baseUri != null && getPageContext() != null) {
        TestPackage userDefinedTestPackage = new UserDefinedTagHandlerTestPackage(
            tagInfo, baseUri + tagInfo.getName(), getPageContext());
        userDefinedTestPackage.executeTest(tagTestContext);
      }

      testContext.getManager().endTestContext(tagTestContext);
    }
  }

}